import React, { useState, useEffect, useRef } from "react";
import ExpandMoreIcon from "@mui/icons-material/ExpandMore";
import ExpandLessIcon from "@mui/icons-material/ExpandLess";
import { UseFormRegisterReturn } from "react-hook-form";

interface option {
  value: string;
  label: string;
}
interface DropdownProps {
  placeholder: string;
  options: option[];
  enableSearch?: boolean;
  onOptionSelect?: (option: option) => void;
  disabled?: boolean; // Atributo para desabilitar o dropdown
  tooltipMessage?: string; // Atributo para exibir uma explicação
  label?: string;
  error?: string;
  name?: string;
  value?: string;
  register?: UseFormRegisterReturn;
}

const Dropdown: React.FC<DropdownProps> = ({
  placeholder,
  options,
  enableSearch = true,
  onOptionSelect,
  disabled = false,
  error,
  label,
  name,
  value,
  tooltipMessage = "",
}) => {
  const [isOpen, setIsOpen] = useState(false);
  const [filteredOptions, setFilteredOptions] = useState<option[]>(options);
  const [selectedOption, setSelectedOption] = useState<option | null>(null);
  const [searchTerm, setSearchTerm] = useState("");
  const [showTooltip, setShowTooltip] = useState(false);
  const dropdownRef = useRef<HTMLDivElement>(null);
  
  useEffect(() => {
    if (value === "") {
      setSelectedOption(null);
    } else if (value) {
      const option = options.find((option) => option.value == value);      
      setSelectedOption(option || null);
    }
  }
  , [value, options]);

  // Handle filtering options based on search term
  useEffect(() => {
    if (searchTerm === "") {
      setFilteredOptions(options);
    } else {
      const filtered = options.filter((option) =>
        option.label.toLowerCase().includes(searchTerm.toLowerCase())
      );
      setFilteredOptions(filtered);
    }
  }, [searchTerm, options]);

  // This function closes the dropdown if the user clicks outside of it
  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (
        dropdownRef.current &&
        !dropdownRef.current.contains(event.target as Node)
      ) {
        setIsOpen(false);
      }
    };
    document.addEventListener("mousedown", handleClickOutside);
    return () => {
      document.removeEventListener("mousedown", handleClickOutside);
    };
  }, [dropdownRef]);

  // This function handles the selection of an option
  const handleOptionSelect = (value: option) => {
    setSelectedOption(value);
    setIsOpen(false);
    // If there is a callback function, call it
    if (onOptionSelect) {
      onOptionSelect(value);
    }
  };

  // Função para alternar a abertura do dropdown, verificando se está desabilitado
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const toggleDropdown = (e: any) => {
    e.preventDefault()
    if (!disabled) {
      setIsOpen(!isOpen);
    } else {
      setShowTooltip(true);
    }
  };

  return (
    <>
    {label && (
        <label htmlFor={name} className="mb-2 text-sm font-medium text-gray-700">
          {label}
        </label>
      )}
      <div
        ref={dropdownRef}
        className="relative w-full"
        onMouseEnter={() => disabled && setShowTooltip(true)}
        onMouseLeave={() => setShowTooltip(false)}
      >
        <button
          onClick={toggleDropdown}
          className={`w-full flex justify-between items-center px-4 py-2 text-sm bg-white text-gray-900 border
            ${disabled ? "bg-gray-300 cursor-not-allowed" : "bg-white"}
            ${
              error ? "border-red-500" : "border-gray-300"
            } rounded-full focus:outline-none`}
        >
          {selectedOption ? selectedOption.label : placeholder}
          <div className="max-h-5">{isOpen ? <ExpandLessIcon /> : <ExpandMoreIcon />}</div>
        </button>

        {/* Tooltip (explicação) */}
        {showTooltip && disabled && (
          <div className="top-full left-0 mt-1 bg-gray-200 text-gray-700 p-2 rounded shadow-lg text-center">
            {tooltipMessage || "Dropdown desabilitado"}
          </div>
        )}

        {isOpen && !disabled && (
          <div className="absolute w-full bg-white shadow-lg rounded-lg mt-2 z-10">
            {/* Search options (opcional) */}
            {enableSearch && (
              <input
                type="text"
                placeholder="Pesquisar..."
                className="w-full p-4 border-b border-gray-200 focus:outline-none"
                value={searchTerm}
                onChange={(e) => setSearchTerm(e.target.value)}
              />
            )}

            {/* Dropdown options */}
            <ul className="max-h-48 overflow-y-auto">
              {filteredOptions.length > 0 ? (
                filteredOptions.map((option, index) => (
                  <li
                    key={index}
                    onClick={() => handleOptionSelect(option)}
                    className="cursor-pointer py-3 px-4 hover:bg-gray-100"
                  >
                    {option.label}
                  </li>
                ))
              ) : (
                <li className="p-4 text-gray-500">Nenhuma opção encontrada</li>
              )}
            </ul>
          </div>
        )}
      </div>
    </>
  );
};

export default Dropdown;
