//eslint-disable @typescript-eslint/no-explicit-any
import { useEffect, useState } from "react";
import CarForm from "../CarForm";
import PieceForm from "../PieceForm";
import AddPieceButton from "../AddPieceButton";
import Button from "../Button";
import { useNavigate } from "react-router-dom";
import { z } from "zod";
import api from "../../services/api";
import { DecodedToken } from "../../dtos/decodedToken";
import { jwtDecode } from "jwt-decode";
import Toast from "../Toast";
import { EditarCarroAdaptado } from "../../pages/editarCarro";
import { PieceData } from "../../pages/cadastroCarro/cadastroCarro";
import Modal from "../Modal";

interface optionDto {
  value: string;
  label: string;
}

const pieceSchema = z.object({
  nome: z
    .number()
    .refine(value => value !== 0, { message: "Nome da peça é obrigatório" }),
  preco: z.any({ message: "Preço da peça é obrigatório" }),
  observacao: z.string().optional(),
});

const carSchema = z.object({
  marca: z.union([z.string(), z.number()]).refine(value => Number(value) !== 0, { message: "Marca é obrigatória" }),
  modelo: z.union([z.string(), z.number()]).refine(value => Number(value) !== 0, { message: "Modelo é obrigatório" }),
  ano: z.string().min(1, { message: "Ano é obrigatório" }),
  valor: z.string().min(1, { message: "Valor é obrigatório" }),
  data_de_compra: z
    .string()
    .min(1, { message: "Data de compra é obrigatória" }),
});

type CarData = z.infer<typeof carSchema>;

export const EditCar = ({
  marca,
  ano,
  valor,
  modelo,
  data_de_compra,
  car_id,
  parts
}: EditarCarroAdaptado
) => {

  const [carData, setCarData] = useState<CarData>({ marca: marca.toString(), ano: ano, valor: valor, modelo: modelo.toString(), data_de_compra: data_de_compra });
  const [pieceForms, setPieceForms] = useState<PieceData[]>(parts);
  const [carErrors, setCarErrors] = useState<any>({});
  const [pieceErrors, setPieceErrors] = useState<any[]>([]);
  const [failedDelete, setFailedDelete] = useState(false);
  const [toastKey, setToastKey] = useState(0);

  const navigate = useNavigate();

  const handleCarInputChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    const { name, value } = e.target;
    setCarData({
      ...carData,
      [name]: value,
    });
  };

  const [carBrandSelected, setCarBrandSelected] = useState(marca);
  const [carBrand, setCarBrand] = useState<optionDto[] | []>([]);
  const [carPart, setCarPart] = useState<optionDto[] | []>([]);
  const [carModel, setCarModel] = useState<optionDto[] | []>([]);
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [deletePieceId, setDeletePieceId] = useState<number>(-1);
  

  useEffect(() => {
    const fetchBrands = async () => {
      const data = await api.get(`/brand`).then((response) => response.data).catch((error) => console.error('Error fetching brands: ', error))
      
      if (!data) {
        return;
      }

      const brandAdated = data.map(
        (item: { enumerator: string; id: string }) => {
          return {
            label: item.enumerator,
            value: item.id,
          };
        }
      );

      setCarBrand(brandAdated);
    };

    const fetchParts = async () => {
      const data = await api
        .get(`/part`)
        .then((response) => response.data)
        .catch((error) => console.error("Error fetching parts: ", error));

      if (!data) {
        return;
      }

      const partAdated = data.map(
        (item: { enumerator: string; id: string }) => {
          return {
            label: item.enumerator,
            value: item.id,
          };
        }
      );

      setCarPart(partAdated);
    };

    fetchBrands();
    fetchParts();
  }, []);

  useEffect(() => {
    const fetchModel = async () => {
      const data = await api
        .get(`/brand/${carBrandSelected}/model`)
        .then((response) => response.data)
        .catch((error) => console.error("Error fetching models: ", error));

      if (!data) {
        return;
      }

      const modelAdated = data.map(
        (item: { enumerator: string; id: string }) => {
          return {
            label: item.enumerator,
            value: item.id,
          };
        }
      );

      setCarModel(modelAdated);
    };
    if (carBrandSelected) {
      fetchModel();
    }
  }, [carBrandSelected]);

  const handleAddPieceClick = () => {
    setPieceForms([...pieceForms, { nome: "", preco: "", id: "", observacao: "", part_status: 1 }]);
    setPieceErrors([...pieceErrors, {}]);
  };

  const handlePieceInputChange = (
    index: number,
    e: React.ChangeEvent<HTMLInputElement>
  ) => {
    const { name, value } = e.target;
    const newPieceForms = [...pieceForms];
    newPieceForms[index][name as keyof PieceData] = value;
    setPieceForms(newPieceForms);
  };
  
  const handleDeletePieceModal = (index: number) => {
    setIsModalOpen(true);
    setDeletePieceId(index);
  }
  
  const handleDeletePiece = async (index: number) => {
    if (pieceForms.length <= 1) return;
    const pieceToDelete = pieceForms[index];
  
    if (pieceToDelete.id) {
      try {
        const response = await api.put(`/part/${pieceToDelete.id}/delete`);
        if (response.status >= 200 && response.status < 300) {
          console.log(`Piece with ID ${pieceToDelete.id} deleted successfully.`);
        } else {
          throw new Error(`Failed to delete part: ${response.statusText}`);
        }
      } catch (error) {
        
        console.error(`Error deleting part with ID ${pieceToDelete.id}:`, error);
        setFailedDelete(true);
        setToastKey (toastKey + 1);
        return; 
      }
    }
    const newPieces = [...pieceForms];
    newPieces.splice(index, 1);
    setPieceForms(newPieces);
  };

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();

    const carValidation = carSchema.safeParse(carData);
    if (!carValidation.success) {
      setCarErrors(carValidation.error.flatten().fieldErrors);
      
      return;
    } else {
      setCarErrors({});
    }

    const newPieceErrors = [...pieceErrors];
    let hasPieceError = false;
    pieceForms.forEach((piece, index) => {
      const pieceValidation = pieceSchema.safeParse(piece);
      if (!pieceValidation.success) {
        hasPieceError = true;
        newPieceErrors[index] = pieceValidation.error.flatten().fieldErrors;
      } else {
        newPieceErrors[index] = {};
      }
    });
    if (hasPieceError) {
      setPieceErrors(newPieceErrors);
      return;
    } else {
      setPieceErrors([]);
    }

    const formattedCar = {
      model: {
        model_id: Number(carData.modelo),
      },
      release_year: carData.ano,
      cost: parseFloat(carData.valor),
      purchase_date: carData.data_de_compra,
    };

    const existingParts = pieceForms.filter(piece => piece.id);
    const newParts = pieceForms.filter(piece => !piece.id);

    try {
      const token = localStorage.getItem("token");

      if (!token) {
        throw new Error("No token found");
      }

      const decoded = jwtDecode<DecodedToken>(token);

      const carResponse = await api.put(
        `/seller/${decoded.id}/cars/${car_id}`,
        formattedCar
      );

      if (carResponse.status < 200 || carResponse.status >= 300) {
        throw new Error(`Failed to submit car data: ${carResponse.statusText}`);
      }

      for (const part of existingParts) {
        const partData = {
          part_type_id: Number(part.nome),
          description: part.observacao,
          price: Number(part.preco),
        };
        const id = parseInt(part.id);

        const partResponse = await api.put(
          `/part/${id}`,
          partData
        );

        if (partResponse.status < 200 || partResponse.status >= 300) {
          console.log("Error updating old parts");
          throw new Error(`Failed to update part data: ${partResponse.statusText}`);
        }
      }

      for (const part of newParts) {
        const partData = {
          car_id: Number(car_id),
          part_type_id: Number(part.nome),
          description: part.observacao,
          price: parseFloat(part.preco),
        };

        const partResponse = await api.post(
          `/part`,
          partData
        );

        if (partResponse.status < 200 || partResponse.status >= 300) {
          console.log("Error adding new parts");
          throw new Error(`Failed to add part data: ${partResponse.statusText}`);
        }
      }

      navigate("/estoque", {
        state: { editedCar: true }
      });

    } catch (error) {
      console.error("Error submitting data:", error);
    }
  };

  return (
    
    <div className="relative flex flex-col justify-center items-center">
      <Modal
          isOpen={isModalOpen}
          title="Excluir peça"
          content="Você tem certeza que deseja excluir essa peça?"
          onCancel={()=>setIsModalOpen(false)}
          onConfirm={()=>{
            handleDeletePiece(deletePieceId)
            setIsModalOpen(false)
          }}
        />
      <Toast key = {toastKey} message="Esta peça não pode ser deletada porque é parte de uma compra" isOpen={failedDelete} isSuccess={false}/>
      <div className="w-full max-w-7xl py-5">
        <div className="bg-white p-8 rounded-lg shadow-lg max-h-[70vh] overflow-y-scroll">
          <div className="w-full max-w-7xl py-5">
            <form onSubmit={handleSubmit}>
            <h2 className="text-2xl font-bold text-black mb-6">
              Editar Carro
            </h2>
              <CarForm
                carData={carData}
                carErrors={carErrors}
                carBrand={carBrand}
                carModel={carModel}
                onCarInputChange={handleCarInputChange}
                onBrandSelect={(option: optionDto) => setCarBrandSelected(Number(option.value))}
              />
              <h2 className="text-2xl font-bold text-black mb-4 mt-6">
              Editar Peças
              </h2>
              {pieceForms.map((pieceForm, index) => (
                <PieceForm
                  key={index}
                  index={index}
                  form={pieceForm}
                  errors={pieceErrors[index]}
                  carPart={carPart}
                  handleInputChange={handlePieceInputChange}
                  handleDelete={handleDeletePieceModal}
                />
              ))}
              
              
            </form>
            <AddPieceButton onClick={handleAddPieceClick} />
          </div>
          
        </div>
        <div className="flex justify-end mt-8">
                <div className="w-80">
                  <Button
                    isFluid={true}
                    customClass="bg-amber-500 hover:bg-amber-600 text-white transition-all duration-300 shadow-md hover:shadow-lg transform hover:translate-x-1"
                    onClick={handleSubmit}
                  >
                    Editar Veículo
                  </Button>
                </div>
              </div>
      </div>
    </div>
  );
};