import { useState, useEffect } from "react";
import { LineChart } from "@mui/x-charts";
import api from "../../services/api";
import {jwtDecode} from "jwt-decode";
import { DecodedToken } from "../../dtos/decodedToken";
import Dropdown from "../../components/Dropdown";


const Dashboard = () => {
    const [chartType, setChartType] = useState<"faturamento" | "custo" | "lucro">(
        "faturamento"
    );

    const colors = {
        faturamento: "#ddc109",
        custo: "#FF0000",
        lucro: "#5cd10d",
    }

    const currentYear = new Date().getFullYear();
    const years = Array.from({ length: 70 }, (_, i) => ({
        value: (currentYear - i).toString(),
        label: (currentYear - i).toString(),
    }));

    interface optionDto {
        label: string;
        value: string;
    }

    const [year, setYear] = useState<number>(currentYear);

    const [data, setData] = useState({
        faturamento: { x: [0], y: [] },
        custo:       { x: [0], y: [] },
        lucro:       { x: [0], y: [] },
    });
    const [loading, setLoading] = useState(true);

    const handleChartChange = (type: "faturamento" | "custo" | "lucro") => {
        setChartType(type);
    };

    const getSellerId = () => {
        const token = localStorage.getItem("token");
        if (!token) {
            throw new Error("No token found");
        }
        const decoded = jwtDecode<DecodedToken>(token);
        return decoded.id;
    };

    const xLabels = ["Jan", "Fev", "Mar", "Abr", "Mai", "Jun", "Jul", "Ago", "Set", "Out", "Nov", "Dez"];

    useEffect(() => {
        const fetchData = async () => {
            setLoading(true);
            try {
                const sellerId = getSellerId();

                const revenueResponse = await api.get(`/seller/${sellerId}/revenue/${year}`);
                const spendingResponse = await api.get(`/seller/${sellerId}/spending/${year}`);

                const revenue = revenueResponse.data.map((val: number) => (val) || 0);
                const spending = spendingResponse.data.map((val: number) => (val) || 0);
                const profit = revenue.map((rev:number, index:number) => rev - (spending[index] || 0));

                setData({
                    faturamento: { x: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12], y: revenue },
                    custo:       { x: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12], y: spending },
                    lucro:       { x: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12], y: profit },
                });
            } catch (error) {
                console.error("Erro ao buscar dados da API:", error);
            } finally {
                setLoading(false);
            }
        };

        fetchData();
    }, [year]);

    if (loading || !data.faturamento.y.every((val) => typeof val === "number")) {
        return (
            <div className="flex justify-center items-center min-h-screen bg-gray-100">
                <p className="text-xl font-semibold">Carregando dados...</p>
            </div>
        );
    }

    if (loading) {
        return (
            <div className="flex justify-center items-center min-h-screen bg-gray-100">
                <p className="text-xl font-semibold">Carregando dados...</p>
            </div>
        );
    }

    return (

            <div className="flex flex-col items-center w-full max-w-4xl p-6 mt-6">
                <div className="flex gap-4 mb-6">
                    <button
                        className={`px-4 py-2 rounded ${
                            chartType === "faturamento"
                                ? "bg-accent text-white"
                                : "bg-gray-200 text-gray-700"
                        }`}
                        onClick={() => handleChartChange("faturamento")}
                    >
                        Faturamento
                    </button>
                    <button
                        className={`px-4 py-2 rounded ${
                            chartType === "custo"
                                ? "bg-accent text-white"
                                : "bg-gray-200 text-gray-700"
                        }`}
                        onClick={() => handleChartChange("custo")}
                    >
                        Custo
                    </button>
                    <button
                        className={`px-4 py-2 rounded ${
                            chartType === "lucro"
                                ? "bg-accent text-white"
                                : "bg-gray-200 text-gray-700"
                        }`}
                        onClick={() => handleChartChange("lucro")}
                    >
                        Lucro
                    </button>

                    <Dropdown
                        placeholder={"Ano atual"}
                        options={years}
                        name="ano"
                        value={year.toString()}
                        onOptionSelect={(option: optionDto) => {
                            setYear(Number(option.value));
                        }
                        }
                    />

                </div>
                <div className="items-center bg-white p-4">
                    <LineChart
                        xAxis={[{ scaleType: 'point', data: xLabels }]}
                        series={[
                            {
                                color: colors[chartType],
                                data: data[chartType].y,
                                label: chartType.charAt(0).toUpperCase() + chartType.slice(1),
                            },
                        ]}
                        width={720}
                        height={400}
                    />
                </div>
            </div>

    );

};

export default Dashboard;
