import React, { useState } from 'react';
import { KeyboardArrowDown, KeyboardArrowUp } from "@mui/icons-material";
import { CanceledPurchasePart } from '../../utils/types/entities';

interface PartType {
  id: number;
  description: string;
  price: number;
  car: {
    model: {
      enumerator: string;
    };
    release_year: string;
  };
  part_type: {
    enumerator: string;
  };
  part_status: {
    enumerator: string;
  };
}

interface PurchasePartType {
  part: PartType;
}


interface PurchaseCardProps {
  index: number;
  description: string;
  purchase_parts: PurchasePartType[];
  created_at: string;
  canceled_purchase_parts: CanceledPurchasePart[];
}

const PurchaseCard: React.FC<PurchaseCardProps> = ({
  index,
  description,
  purchase_parts,
  created_at,
  canceled_purchase_parts,
}) => {
  const [isExpanded, setIsExpanded] = useState(false);
  
  const totalPrice = purchase_parts.reduce((sum, pp) => sum + pp.part.price, 0);
  
  const formatDate = (dateString: string) => {
    const date = new Date(dateString);
    return date.toLocaleDateString('pt-BR');
  };

  const getStatusColor = (status: string) => {
    switch (status) {
      case 'available':
        return 'bg-red-500 text-stone-900';
      case 'ordered':
        return 'bg-yellow-100 text-yellow-800';
      case 'sold':
        return 'bg-green-100 text-green-800';
      default:
        return 'bg-gray-100 text-gray-800';
    }
  };

  const formatStatus = (status: string) => {
    const statusMap: { [key: string]: string } = {
      'available': 'Cancelada',
      'ordered': 'Encomendada',
      'sold': 'Entregue',
    };
    return statusMap[status] || status;
  };
  
  return (
    <div className={`w-full border rounded-lg p-4 shadow-sm ${index !== 0 ? 'mt-4' : ''}`}>
      <div 
        className="cursor-pointer"
        onClick={() => setIsExpanded(!isExpanded)}
      >
        <div className="flex justify-between mb-2">
          <h3 className="font-semibold">{description}</h3>
          <span className="font-semibold text-gray-600">
            R$ {totalPrice.toLocaleString('pt-BR')}
          </span>
        </div>
        <div className="flex justify-between text-sm text-gray-500">
          <span>Data da compra</span>
          <span className="font-semibold">{formatDate(created_at)}</span>
        </div>
        <div className="flex justify-center mt-4">
          {isExpanded ? (
            <KeyboardArrowUp className="text-gray-500" />
          ) : (
            <KeyboardArrowDown className="text-gray-500" />
          )}
        </div>
      </div>

      {isExpanded && (
        <div className="mt-4 space-y-4 border-t pt-4">
          {purchase_parts?.map((pp, partIndex) => (
            <div 
              key={pp.part.id}
              className={`p-3 rounded-lg ${partIndex !== 0 ? 'border-t pt-4' : ''}`}
            >
              <div className="flex justify-between items-start mb-2">
                <span className="font-semibold">{pp.part.description}</span>
                <div className="flex flex-col items-end">
                  <span className="font-semibold text-gray-600">
                    R$ {pp.part.price.toLocaleString('pt-BR')}
                  </span>
                  <span className={`text-xs px-2 py-1 rounded-full mt-1 ${getStatusColor(pp.part.part_status.enumerator)}`}>
                    {formatStatus(pp.part.part_status.enumerator)}
                  </span>
                </div>
              </div>
              <div className="flex justify-between text-sm text-gray-500">
                <span>Tipo de peça</span>
                <span>Modelo do carro</span>
              </div>
              <div className="flex justify-between text-sm mt-1">
                <span className="font-semibold">{pp.part.part_type.enumerator}</span>
                <span className="font-semibold">{pp.part.car.model.enumerator}</span>
              </div>
            </div>
          ))}
          {canceled_purchase_parts?.map((pp, partIndex) => (
            <div 
              key={pp.part.id}
              className={`p-3 rounded-lg ${partIndex !== 0 ? 'border-t pt-4' : ''}`}
            >
              <div className="flex justify-between items-start mb-2">
                <span className="font-semibold">{pp.part.description}</span>
                <div className="flex flex-col items-end">
                  <span className="font-semibold text-gray-600">
                    R$ {pp.part.price.toLocaleString('pt-BR')}
                  </span>
                  <span className={`text-xs px-2 py-1 rounded-full mt-1 ${getStatusColor("available")}`}>
                    {formatStatus("available")}
                  </span>
                </div>
              </div>
              <div className="flex justify-between text-sm text-gray-500">
                <span>Tipo de peça</span>
                <span>Modelo do carro</span>
              </div>
              <div className="flex justify-between text-sm mt-1">
                <span className="font-semibold">{pp.part.part_type.enumerator}</span>
                <span className="font-semibold">{pp.part.car.model.enumerator}</span>
              </div>
            </div>
          ))}
        </div>
      )}
    </div>
  );
};

export default PurchaseCard;