import React, { useEffect, useRef, useState } from "react";

import { CircularProgress } from "@mui/material";
import { HourglassEmpty } from "@mui/icons-material";

import { jwtDecode } from "jwt-decode";

import api from "../../services/api";
import { DecodedToken } from "../../dtos/decodedToken";
import PurchaseCard from "../../components/PurchaseCard";
import { Purchase } from "../../utils/types/entities";
import Toast from "../../components/Toast";
import { useLocation } from "react-router-dom";

const defaultState = {
  purchases: [] as Purchase[],
  loading: {
    purchases: true,
  },
};

const BuyerPurchases: React.FC = () => {
  const [state, setState] = useState(defaultState);

  const isFirstRender = useRef(true);
  useEffect(() => {
    if (isFirstRender.current) {
      isFirstRender.current = false;
      handleFetchPurchases();
    }
  }, []);

  const handleFetchPurchases = async () => {
    try {
      const token = localStorage.getItem("token");
      if (!token) {
        throw new Error("No token found");
      }
      const decoded = jwtDecode<DecodedToken>(token);
      const data = await api
        .get(`buyer/${decoded.id}/purchases`)
        .then((response) => response.data);
      setState((prevState) => ({
        ...prevState,
        purchases: data,
        loading: { ...prevState.loading, purchases: false },
      }));
    } catch (error) {
      console.error("Error fetching purchases: ", error);
      setState((prevState) => ({
        ...prevState,
        loading: { ...prevState.loading, purchases: false },
      }));
    }
  };
  const location = useLocation();
  
  const { createdPurchase } = location.state || {};

  return (
    <div className="flex flex-col items-center h-full bg-orange-500">
      <Toast message="Pedido realizado com sucesso" isOpen={createdPurchase} />
      <div className="w-full max-w-md flex-grow flex flex-col items-center p-4 overflow-hidden h-full">
        <div className="mb-6 mt-4">
          <img
            src="/logoprojeto.png"
            alt="Logo"
            className="rounded-full w-24 h-24"
          />
        </div>
        <h2 className="text-2xl font-normal text-center text-white mb-6">
          Histórico de Compras
        </h2>
        <div className="w-full max-w-md bg-white rounded-lg shadow-lg p-6 flex flex-col flex-grow overflow-hidden">
          {state.loading.purchases ? (
            <div className="flex justify-center my-8 text-orange-500">
              <CircularProgress />
            </div>
          ) : (
            <div className="w-full flex-grow overflow-y-auto no-scrollbar mt-4 h-full">
              {!state.purchases.length ? (
                <div className="flex flex-col items-center justify-center h-full text-center text-gray-500">
                  <HourglassEmpty style={{ fontSize: 48 }} />
                  <span>Realize uma compra para visualizar o histórico</span>
                </div>
              ) : (
                state.purchases.map((purchase, index) => (
                  <PurchaseCard
                    key={purchase.id}
                    index={index}
                    description={purchase.description}
                    purchase_parts={purchase.purchase_parts}
                    created_at={purchase.created_at}
                    canceled_purchase_parts={purchase.canceled}
                  />
                ))
              )}
            </div>
          )}
        </div>
      </div>
    </div>
  );
};

export default BuyerPurchases;
