import { useForm } from "react-hook-form";
import Input from "../../components/Input";
import { z } from "zod";
import { zodResolver } from "@hookform/resolvers/zod";
import axios from "axios";
import { useEffect, useState } from "react";
import { jwtDecode } from "jwt-decode";
import { DecodedToken } from "../../dtos/decodedToken";
import { useNavigate } from "react-router-dom";

const createUserFormSchema = z.object({
  email: z
      .string()
      .nonempty("O email é obrigatório")
      .email("Formato de email inválido")
      .toLowerCase(),
  password: z.string().min(4, "A senha precisa de 4 caracteres"),
});

type CreateUserFormData = z.infer<typeof createUserFormSchema>;

const Login = () => {
  const {
    register,
    handleSubmit,
    formState: { errors },
  } = useForm<CreateUserFormData>({
    resolver: zodResolver(createUserFormSchema),
  });
  
  useEffect(() => {
    const token = localStorage.getItem("token");
    if(token) {
      localStorage.removeItem("token");
    }
  }, []);
  
  const navigate = useNavigate();
  
  const [errorMessage, setErrorMessage] = useState<string | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(false);

  const loginUser = async (data: CreateUserFormData) => {
    try {
      setIsLoading(true); // Inicia o estado de carregamento
      const response = await axios.post(`/api/auth/sign_in`, data);

      localStorage.setItem("token", response.data.accessToken);
      const decoded = jwtDecode<DecodedToken>(response.data.accessToken);
      
  
      if(decoded?.role === "admin") {
        navigate("/admin");
      } else {
        navigate("/home");
      }
      
    } catch (error) {
      if (axios.isAxiosError(error)) {
        setErrorMessage(error.response?.data?.message || "Falha no login");
      } else {
        setErrorMessage("Falha no login");
      }
    } finally {
      setIsLoading(false); // Finaliza o estado de carregamento
    }
  };

  const onSubmit = (data: CreateUserFormData) => {
    setErrorMessage(null);
    loginUser(data);
  };

  return (
      <div className="flex h-screen justify-center items-center bg-amber-50">
        <div className="flex w-[80%] h-[80%] shadow-lg rounded-lg overflow-hidden">
          <div className="w-1/2 bg-white flex flex-col justify-center items-center p-10">
            {/*Primeira metade da tela*/}
            <h1 className="text-3xl font-bold mb-14">Login</h1>
            <form onSubmit={handleSubmit(onSubmit)} className="w-full max-w-sm flex flex-col space-y-6">
              <div>
                {/* Campo Email */}
                <Input
                    type="email"
                    label="Email"
                    name="email"
                    register={register("email")}
                    placeholder="Digite seu email"
                    icon="/logoemail.svg"
                />
                {errors.email && (
                    <p className="text-red-500 text-xs mt-1">{errors.email.message}</p>
                )}
              </div>
              <div>
                <Input
                    type="password"
                    name="password"
                    label="Senha"
                    register={register("password")}
                    placeholder="Digite sua senha"
                    icon="/logosenha.svg"
                />
                {errors.password && (
                    <p className="text-red-500 text-xs mt-1">{errors.password.message}</p>
                )}
              </div>
              <button
                  type="submit"
                  className="w-full bg-orange-500 text-white py-2 px-4 rounded-full hover:bg-orange-600 transition mt-60 mb-2"
                  disabled={isLoading}
              >
                {isLoading ? "Carregando..." : "Log in"}
              </button>
            </form>
            {/* Exibir erro de login */}
            {errorMessage && <p className="text-red-500 text-sm mt-4">{errorMessage}</p>}
          </div>

          <div className="w-1/2 bg-orange-500 flex flex-col justify-center items-center">
            {/*Segunda metade da tela*/}
            <div className="rounded-lg mb-16">
              <img src="/logoprojeto.png" alt="Logo" />
            </div>
            <div className="">
              <h2 className="text-2xl font-bold text-center text-white">
                Bem-vindo <br /> de volta!
              </h2>
            </div>
          </div>
        </div>
      </div>
  );
};

export default Login;
