import { useForm } from "react-hook-form";
import Input from "../../components/Input";
import { z } from "zod";
import { zodResolver } from "@hookform/resolvers/zod";
import axios from "axios";
import { useEffect, useState } from "react";
import { jwtDecode } from "jwt-decode";
import { DecodedToken } from "../../dtos/decodedToken";
import {Link, useLocation, useNavigate} from "react-router-dom";
import api from "../../services/api";
import Toast from "../../components/Toast";

const createUserFormSchema = z.object({
  email: z
    .string()
    .nonempty("O email é obrigatório")
    .email("Formato de email inválido")
    .toLowerCase(),
  password: z.string().min(4, "A senha precisa de 4 caracteres"),
});

type CreateUserFormData = z.infer<typeof createUserFormSchema>;

const Login = () => {
  const navigate = useNavigate();

  const {
    register,
    handleSubmit,
    formState: { errors },
  } = useForm<CreateUserFormData>({
    resolver: zodResolver(createUserFormSchema),
  });

  useEffect(() => {
    const token = localStorage.getItem("token");
    if (token) {
      localStorage.removeItem("token");
    }
  }, []);

  const [errorMessage, setErrorMessage] = useState<string | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(false);

  const loginUser = async (data: CreateUserFormData) => {
    try {
      setIsLoading(true);
      const response = await api.post(`/auth/sign_in`, data);
      localStorage.setItem("token", response.data.accessToken);
      const decoded = jwtDecode<DecodedToken>(response.data.accessToken);
      if(decoded?.role !== "buyer") {
        throw new Error("Usuário não é um mecânico");
      }
      navigate("/mecanico/busca");
      

    } catch (error) {
      if (axios.isAxiosError(error)) {
        setErrorMessage(error.response?.data?.message || "Falha no login");
      } else {
        setErrorMessage("Falha no login");
      }
    } finally {
      setIsLoading(false);
    }
  };

  const onSubmit = (data: CreateUserFormData) => {
    setErrorMessage(null);
    loginUser(data);
  };

  const location = useLocation();
  const { createdBuyer } = location.state || {};

  return (
    <div className="flex h-screen flex-col items-center bg-orange-500">
      <Toast message="Usuário cadastrado com sucesso" isOpen={createdBuyer} />
      <div className="mb-12 mt-10">
        <img src="/logoprojeto.png" alt="Logo" className="rounded-full size-24" />
      </div>
      <div className="flex w-[80%] h-fit shadow-lg rounded-lg overflow-hidden bg-white flex-col justify-center items-center p-10">
        {/*Primeira metade da tela*/}
        <h1 className="text-3xl font-bold mb-3">Entrar</h1>
        <div className="mb-7 flex gap-1">
          <h1 className="text-xs font-boldtext-neutral-500">Não tem uma conta? </h1>
          <Link to="/mecanico-cadastro" className="underline text-xs text-blue-400 font-boldtext-neutral-500">
            Cadastre-se
          </Link>
        </div>
        <form onSubmit={handleSubmit(onSubmit)} className="w-full max-w-md flex flex-col space-y-4">
          <div>
            {/* Campo Email */}
            <Input
              type="email"
              label="Email"
              name="email"
              register={register("email")}
              placeholder="Digite seu email"
              borderRadius="rounded-md"
              icon="logoemail.svg"
            />
            {errors.email && (
              <p className="text-red-500 text-xs mt-1">{errors.email.message}</p>
            )}
          </div>
          <div>
            <Input
              type="password"
              name="password"
              label="Senha"
              register={register("password")}
              placeholder="Digite sua senha"
              icon="/olhoSenha.svg"
              borderRadius="rounded-md"
            />

            {errors.password && (
              <p className="text-red-500 text-xs mt-1">{errors.password.message}</p>
            )}
          </div>

          <button
            type="submit"
            className="w-full bg-orange-500 text-white py-2 px-4 rounded-lg hover:bg-orange-600 transition mt-60 mb-2"
            disabled={isLoading}
          >
            {isLoading ? "Carregando..." : "Log in"}
          </button>
        </form>
        {/* Exibir erro de login */}
        {errorMessage && <p className="text-red-500 text-sm mt-4">{errorMessage}</p>}
      </div>

    </div>
  );
};

export default Login;
