/* eslint-disable @typescript-eslint/no-explicit-any */
import Header from "../../components/Header";
import { CarCard } from "../../components/CarCard";
import { ContainerEstoque } from "../../components/ContainerEstoque";
import ContainerPedidos from "../../components/ContainerPedidos";
import ContainerFerroVelho from "../../components/ContainerFerroVelho";
import api from "../../services/api";
import { DecodedToken } from "../../dtos/decodedToken";
import { jwtDecode } from "jwt-decode";
import { useEffect, useState } from "react";
import { useLocation, useNavigate } from "react-router-dom";
import { Seller } from "../../utils/types/entities";
import Toast from "../../components/Toast";

interface Car {
  id: string;
  carName: string;
  costs: number;
  purchaseDate: string;
  registeredParts: number;
  soldParts: number;
  expectedReturn: number;
  currentReturn: number;
}

interface Order {
  id: string;
  item: string;
  price: number;
  date: string;
  status: number;
  buyer: string;
  part_id: number;
}

function useQuery() {
  return new URLSearchParams(useLocation().search);
}

function MeuEstoque() {
  const [stock, setStock] = useState<Car[]>([]);
  const [purchases, setPurchases] = useState<Order[]>([]);
  const [seller, setSeller] = useState<Partial<Seller>>({} as Seller);
  const query = useQuery();
  const navigate = useNavigate();
  const [isLoading, setIsLoading] = useState(false);
  
  const location = useLocation();
  const { editedCar } = location.state || {};
  
  const token = localStorage.getItem("token");
  if (!token) {
    throw new Error("No token found");
  }

  let sellerId = "";
  const queryParam = query.get("seller");
  const decoded = jwtDecode<DecodedToken>(token);

  if (queryParam) {
    sellerId = queryParam;  
  } else {
    sellerId = decoded.id;
  }

  const getStock = async () => {
    try {
      const response = await api.get(`/seller/${sellerId}/cars`);
      
      const adjustedData = response.data.map((car: any) => ({
        id: car.id,
        carName: `${car.model.brand.enumerator} ${car.model.enumerator}`,
        costs: car.cost,
        currentReturn: car.parts.reduce((acc: number, part: any) => {
          if (part.part_status_id === 3) {
            return acc + part.price;
          }
          return acc;
        }
        , 0),
        purchaseDate: new Date(car.purchase_date).toLocaleDateString("pt-BR"),
        registeredParts: car.parts.length,
        expectedReturn: car.parts.reduce((acc: number, part: any) => acc + part.price, 0),
        soldParts: car.parts.filter((part: any) => part.part_status_id === 3).length,
      }));

      setStock(adjustedData);
    } catch (error) {
      console.error("Error fetching stock data:", error);
    }
  };
  
  const getPurchases = async () => {
    try {
      const response = await api.get(`/seller/${sellerId}/purchases`);
      
      const adjustedOrders = response.data.map((order: any) => ({
        id: order.part.id,
        item: `${order.part.part_type.enumerator} (${order.part.car.model.brand.enumerator} ${order.part.car.model.enumerator})`,
        price: order.part.price,
        date: new Date(order.purchase.created_at).toLocaleDateString("pt-BR"),
        status: order.part.part_status.id,
        buyer: order.purchase.buyer.company_name,
        part_id: order.part.id
      }));

      setPurchases(adjustedOrders);
    } catch (error) {
      console.error("Error fetching orders:", error);
    }
  };

  const getSeller = async () => {
    try {
      const response = await api.get(`/seller/${queryParam}`);
      const adjustedData: Partial<Seller> = {
        user_id: response.data.user_id,
        company_name: response.data.company_name,
        seller_code: response.data.seller_code,
        document_number: response.data.document_number, 
        trading_name: response.data.trading_name,
        phone_number: response.data.phone_number,
        email: response.data.email,
        phone_area_code: response.data.phone_area_code,
        phone_country_code: response.data.phone_country_code,
      };

      setSeller(adjustedData);
      console.log("Seller:", JSON.parse(JSON.stringify(response)));
      console.log("Adjusted Data:", JSON.parse(JSON.stringify(adjustedData)));
    } catch (error) {
      console.error("Error fetching sellers:", error);
    }
  }

  useEffect(() => {
    getStock();
    if (decoded.role == "admin") {
      getSeller();
    } else {
      getPurchases();
    }
  }, []);
  
  const handleConfirm = async (orderId: string) => {
    
    try {
      // Aqui você pode fazer a requisição para a API para confirmar a venda
      setIsLoading(true);
      const response = await api.put(`/purchase/${orderId}/update_purchase`);
      setIsLoading(false);
      
      console.log("Venda confirmada com sucesso:", response.data);
      getPurchases();
      getStock();
            
    } catch (error) {
      console.error("Ocorreu um erro ao tentar confirmar a venda:",
        error
      );
    }
  };
  
  const handleCancel = async (partId: number) => {
    
    try {
      // Aqui você pode fazer a requisição para a API para cancelar a venda
      setIsLoading(true);
      await api.delete(`/seller/${partId}/delete_purchase`);
      setIsLoading(false);
      
      getPurchases();
      getStock();
      
    } catch (error) {
      console.error("Ocorreu um erro ao tentar cancelar a venda:",
        error
      );
    }
  };

  return (
    <div className="bg-orange-100 min-h-screen">
      <Toast message="Edição de carro realizada com sucesso" isOpen={editedCar} />
      <Header
        leftComponent={
          <img
            src="logo.svg"
            alt="Logo"
            className="w-24 h-24"
            onClick={() => navigate(-1)}
            style={{ cursor: "pointer" }}
          />
        }
        rightComponent={
          <h1 className="text-black items-center">Bom pra carro</h1>
        }
        customClass="bg-orange-100"
      />
      {decoded.role === "admin" ?
      <ContainerFerroVelho isLoading={isLoading} seller={seller} />
      :
      <ContainerPedidos isLoading={isLoading} orders={purchases} handleConfirmation={handleConfirm} handleCancel={handleCancel} />
      }
      <ContainerEstoque isAdmin={decoded.role === "admin"}>
        {stock.length === 0 ? (
          <h1 className="text-center text-gray-600">
            Você ainda não possui carros cadastrados
          </h1>
        ) : (
          stock.map((car) => (
            <CarCard
              isAdmin={decoded.role === "admin"}
              key={car.id}
              idCar={car.id}
              carName={car.carName}
              costs={car.costs}
              currentReturn={car.currentReturn}
              purchaseDate={car.purchaseDate}
              registeredParts={car.registeredParts}
              expectedReturn={car.expectedReturn}
              soldParts={car.soldParts || 0}
            />
          ))
        )}
      </ContainerEstoque>
    </div>
  );
}

export default MeuEstoque;
