import React, { useEffect, useRef, useState } from "react";
import { useNavigate } from "react-router-dom";
import api from "../../services/api";
import { jwtDecode } from "jwt-decode";
import { DecodedToken } from "../../dtos/decodedToken";
import Header from "../../components/Header";
import {
  formatStringIsoDateYYYMMDD,
  formatStringIsoDateYYYY,
} from "../../utils/handlers/date";
import { Car } from "../../utils/types/entities";
import Graphics from "../../components/Graphics";

const defaultState = {
  purchaseCars: [] as Car[],
  soldCars: [] as Car[],
  loading: {
    purchaseCars: true,
    soldCars: true,
  },
};

const Reports: React.FC = () => {
  const navigate = useNavigate();
  const [state, setState] = useState(defaultState);
  const isFirstRender = useRef(true);

  useEffect(() => {
    if (isFirstRender.current) {
      isFirstRender.current = false;
      handleFetchPurchaseCars();
      handleFetchSoldCars();
    }
  }, []);

  const calculateTotalPartsPrice = (car: Car) => {
    return car.parts.reduce((total, part) => total + part.price, 0);
  };

  const handleFetchPurchaseCars = async () => {
    try {
      const token = localStorage.getItem("token");
      if (!token) {
        throw new Error("No token found");
      }
      const decoded = jwtDecode<DecodedToken>(token);
      const data = await api
        .get(`seller/${decoded.id}/cars`)
        .then((response) => response.data);
      setState((prevState) => ({
        ...prevState,
        purchaseCars: data,
        loading: { ...prevState.loading, purchaseCars: false },
      }));
    } catch (error) {
      console.error("Error fetching purchase cars: ", error);
      setState((prevState) => ({
        ...prevState,
        loading: { ...prevState.loading, purchaseCars: false },
      }));
    }
  };

  const handleFetchSoldCars = async () => {
    try {
      const token = localStorage.getItem("token");
      if (!token) {
        throw new Error("No token found");
      }
      const decoded = jwtDecode<DecodedToken>(token);
      const data = await api
        .get(`seller/${decoded.id}/cars?status=sold`)
        .then((response) => response.data);
      setState((prevState) => ({
        ...prevState,
        soldCars: data,
        loading: { ...prevState.loading, soldCars: false },
      }));
    } catch (error) {
      console.error("Error fetching sold cars: ", error);
      setState((prevState) => ({
        ...prevState,
        loading: { ...prevState.loading, soldCars: false },
      }));
    }
  };

  return (
    <div className="bg-orange-100 min-h-screen">
      <Header
        leftComponent={
          <img
            src="logo.svg"
            alt="Logo"
            className="w-24 h-24"
            onClick={() => navigate(-1)}
            style={{ cursor: "pointer" }}
          />
        }
        rightComponent={
          <h1 className="text-black items-center">Bom pra carro</h1>
        }
        customClass="bg-orange-100"
      />

      <div className="max-w-6xl mx-auto px-4 py-8">

        <div className="bg-white flex flex-col items-center justify-center shadow-md p-6 rounded-lg mb-6">
          <h2 className="text-xl font-semibold mr-auto mb-4">Gráfico de Rendimentos</h2>
          <Graphics />

        </div>

        <div className="bg-white rounded-lg shadow-md p-6 mb-6">
          <h2 className="text-xl font-semibold mb-4">Carros Comprados</h2>
          <div className="overflow-x-auto">
            <table className="w-full">
              <thead>
              <tr className="border-b">
                <th className="text-left py-2">Modelo</th>
                <th className="text-left py-2">Marca</th>
                <th className="text-right py-2">Ano</th>
                <th className="text-right py-2">Preço</th>
                <th className="text-right py-2">Data de Compra</th>
              </tr>
              </thead>
              <tbody>
              {state.purchaseCars.map((car) => (
                  <tr key={car.id} className="border-b hover:bg-orange-50">
                    <td className="py-2">{car.model.enumerator}</td>
                    <td className="py-2">{car.model.brand.enumerator}</td>
                    <td className="py-2 text-right">
                      {formatStringIsoDateYYYY(car.release_year)}
                    </td>
                    <td className="py-2 text-right">
                      {car.cost.toLocaleString("pt-BR", {
                        style: "currency",
                        currency: "BRL",
                      })}
                    </td>
                    <td className="py-2 text-right">
                      {formatStringIsoDateYYYMMDD(car.purchase_date)}
                    </td>
                  </tr>
              ))}
              </tbody>
            </table>
          </div>
        </div>

        <div className="bg-white rounded-lg shadow-md p-6">
          <h2 className="text-xl font-semibold mb-4">Carros Vendidos</h2>
          <div className="overflow-x-auto">
            <table className="w-full">
              <thead>
              <tr className="border-b">
                <th className="text-left py-2">Modelo</th>
                <th className="text-left py-2">Marca</th>
                <th className="text-right py-2">Ano</th>
                <th className="text-right py-2">Valor Total das Peças</th>
                <th className="text-right py-2">Data de Venda</th>
              </tr>
              </thead>
              <tbody>
              {state.soldCars.map((car) => (
                  <tr key={car.id} className="border-b hover:bg-orange-50">
                    <td className="py-2">{car.model.enumerator}</td>
                    <td className="py-2">{car.model.brand.enumerator}</td>
                    <td className="py-2 text-right">
                      {formatStringIsoDateYYYY(car.release_year)}
                    </td>
                    <td className="py-2 text-right">
                      {calculateTotalPartsPrice(car).toLocaleString("pt-BR", {
                        style: "currency",
                        currency: "BRL",
                      })}
                    </td>
                    <td className="py-2 text-right">
                      {formatStringIsoDateYYYMMDD(car.sale_date)}
                    </td>
                  </tr>
              ))}
              </tbody>
            </table>
          </div>
        </div>


      </div>
    </div>
  );
};

export default Reports;
