import React, { useEffect, useState } from 'react';
import { useNavigate, useParams } from 'react-router-dom';
import CarPartsTable from '../../components/CarPartsTable';
import BackButton from '../../components/BackButton';
import api from '../../services/api';
import { jwtDecode } from 'jwt-decode';

interface DecodedToken {
  id: string;
  // Outros campos, se houver
}

interface CarPart {
  id: string;
  name: string;
  status: 'Em estoque' | 'Encomendada' | 'Vendida';
  price: number; // O preço é um número
  date: string;
}

interface CarPartsTableProps {
  parts: CarPart[];
  carName: string;
  carYear: string;
  purchaseDate: string;
  cost: number; // O custo é um número
  expectedReturn: number; // Lucro esperado como número
  currentReturn: number; // Balanço atual como número
  totalPartsValue: number; // Valor total das peças
}

const TabelaCarro: React.FC = () => {
  const navigate = useNavigate();
  const { car_id } = useParams<{ car_id: string }>();
  const [carData, setCarData] = useState<CarPartsTableProps | null>(null);

  const handleBackClick = () => {
    navigate(-1);
  };

  const handleFetchParts = async () => {
    try {
      const token = localStorage.getItem('token');
      if (!token) {
        throw new Error('No token found');
      }

      const decoded: DecodedToken = jwtDecode(token);
      const userId = decoded.id;

      const response = await api.get(`seller/${userId}/car/${car_id}/parts`);
      const data = response.data;

      // Converter o custo do carro para reais
      const cost = data.cost;

      // Mapear as peças, convertendo os preços para reais e ajustando os status
      const parts: CarPart[] = data.parts.map((part: any) => {
        // Converter o status
        
        let status = part.part_status.enumerator;
        if (status === 'available' || status === 'Disponível') {
          status = 'Em estoque';
        } else if (status === 'sold' || status === 'Vendido') {
          status = 'Vendida';
        } else if (status === "ordered") {
          status = "Encomendada"
        }

        return {
          id: part.id.toString(),
          name: `${part.part_type.enumerator} (${data.model.brand.enumerator} ${data.model.enumerator})`,
          status: status,
          price: part.price, // Converter para reais
          date: new Date(part.updated_at).toLocaleDateString('pt-BR'),
        };
      });

      // Calcular o total dos preços das peças
      const totalPartsPrice = parts.reduce(
        (acc: number, part: CarPart) => acc + part.price,
        0
      );

      // Calcular o total dos preços das peças vendidas
      const soldPartsTotal = parts.reduce(
        (acc: number, part: CarPart) =>
          part.status === 'Vendida' ? acc + part.price : acc,
        0
      );

      // Calcular o lucro esperado (total das peças - custo do carro)
      const expectedReturn = totalPartsPrice - cost;

      // Calcular o balanço atual (total das peças vendidas - custo do carro)
      const currentReturn = soldPartsTotal - cost;

      // Valor total das peças
      const totalPartsValue = totalPartsPrice;

      // Criar o objeto carData com os valores calculados
      const carData: CarPartsTableProps = {
        carName: `${data.model.brand.enumerator} ${data.model.enumerator}`,
        carYear: new Date(data.release_year).getFullYear().toString(),
        purchaseDate: new Date(data.purchase_date).toLocaleDateString('pt-BR'),
        cost: cost,
        expectedReturn: expectedReturn,
        currentReturn: currentReturn,
        totalPartsValue: totalPartsValue,
        parts: parts,
      };

      setCarData(carData);
    } catch (error) {
      console.error('Error fetching purchases: ', error);
    }
  };

  // Função para atualizar os dados após mudança de status
  const handleStatusChange = () => {
    handleFetchParts();
  };

  useEffect(() => {
    handleFetchParts();
  }, []);

  return (
    <div className="min-h-screen flex flex-col items-center bg-[#FFF4EA] px-4 py-6">
      <div className="self-start ml-4 mb-4">
        <BackButton onConfirmBack={handleBackClick} showModalOnBack={false} />
      </div>

      <div className="flex justify-center mb-6">
        <img src="/logo.svg" alt="Logo" className="w-16 h-auto" />
      </div>

      {carData ? (
        <CarPartsTable
          carName={carData.carName}
          carYear={carData.carYear}
          purchaseDate={carData.purchaseDate}
          cost={carData.cost}
          expectedReturn={carData.expectedReturn}
          currentReturn={carData.currentReturn}
          totalPartsValue={carData.totalPartsValue}
          carId={car_id as string}
          parts={carData.parts}
          onStatusChange={handleStatusChange} // Passa a função para atualizar o estado
        />
      ) : (
        <p>Carregando dados...</p>
      )}
    </div>
  );
};

export default TabelaCarro;
