/* eslint-disable @typescript-eslint/no-explicit-any */
import React, { useState, useEffect, useRef } from "react";

import Select from "react-select";
import { HourglassEmpty, Tune } from "@mui/icons-material";
import { CircularProgress } from "@mui/material";

import api from "../../services/api";
import CarCardSearch from "../../components/CarCardSearch";
import { Part } from "../../utils/types/entities";
import { useBuyerStore } from "../../store";

interface VehiclePartSearchState {
  brandLoading: boolean;
  brandOptions: { value: string; label: string }[];
  carBrand: { value: string; label: string } | null;
  modelLoading: boolean;
  modelOptions: { value: string; label: string }[];
  carModel: { value: string; label: string } | null;
  yearFrom: { value: string; label: string } | null;
  yearTo: { value: string; label: string } | null;
  partLoading: boolean;
  partOptions: { value: string; label: string }[];
  carPart: { value: string; label: string } | null;
  searchLoading: boolean;
  partsResult: Part[];
}

const defaultState: VehiclePartSearchState = {
  brandLoading: false,
  brandOptions: [],
  carBrand: null,
  modelLoading: false,
  modelOptions: [],
  carModel: null,
  yearFrom: null,
  yearTo: null,
  partLoading: false,
  partOptions: [],
  carPart: null,
  searchLoading: false,
  partsResult: [],
};

const currentYear = new Date().getFullYear();
const years = Array.from({ length: 70 }, (_, i) => ({
  value: (currentYear - i).toString(),
  label: (currentYear - i).toString(),
}));

const VehiclePartSearch: React.FC = () => {
  const [state, setState] = useState(defaultState);
  const [toggleFilter, setToggleFilter] = useState(true);

  const isFirstRender = useRef(true);
  useEffect(() => {
    if (isFirstRender.current) {
      handleFetchBrands();
      handleFetchParts(); // Carregar peças disponíveis
      isFirstRender.current = false;
    }
  }, []);

  const handleFetchBrands = async () => {
    setState((prevState) => ({ ...prevState, brandLoading: true }));
    try {
      const data = await api.get(`/brand`).then((response) => response.data);
      setState((prevState) => ({
        ...prevState,
        brandOptions: data.map((brand: any) => ({
          value: brand.id,
          label: brand.enumerator,
        })),
        brandLoading: false,
      }));
    } catch (error) {
      console.error("Error fetching brands: ", error);
      setState((prevState) => ({ ...prevState, brandLoading: false }));
    }
  };

  const handleFetchParts = async () => {
    setState((prevState) => ({ ...prevState, partLoading: true }));
    try {
      const data = await api.get(`/part`).then((response) => response.data);
      setState((prevState) => ({
        ...prevState,
        partOptions: data.map((part: any) => ({
          value: part.id,
          label: part.enumerator, 
        })),
        partLoading: false,
      }));
    } catch (error) {
      console.error("Error fetching parts: ", error);
      setState((prevState) => ({ ...prevState, partLoading: false }));
    }
  };

  const onSelectedBrand = async (brand: { value: string; label: string }) => {
    setState((prevState) => ({ ...prevState, modelLoading: true }));
    try {
      const data = await api
        .get(`/brand/${brand.value}/model`)
        .then((response) => response.data);
      setState((prevState) => ({
        ...prevState,
        carBrand: brand,
        modelOptions: data.map((model: any) => ({
          value: model.id,
          label: model.enumerator,
        })),
        carModel: null,
        modelLoading: false,
      }));
    } catch (error) {
      console.error("Error fetching models: ", error);
      setState((prevState) => ({ ...prevState, modelLoading: false }));
    }
  };

  const handleSearchParts = async () => {
    setState((prevState) => ({ ...prevState, searchLoading: true }));
    const params = {
      ...(state.carBrand?.value && { brand_id: state.carBrand.value }),
      ...(state.carModel?.value && { model_id: state.carModel.value }),
      ...(state.yearFrom?.value && { year_from: state.yearFrom.value }),
      ...(state.yearTo?.value && { year_to: state.yearTo.value }),
      ...(state.carPart?.value && { part_type_id: state.carPart.value }),
    };
    try {
      const data = await api
        .get(`/part/available`, { params })
        .then((response) => response.data);
      setState((prevState) => ({
        ...prevState,
        partsResult: data,
        searchLoading: false,
      }));
    } catch (error) {
      console.error("Error fetching parts: ", error);
      setState((prevState) => ({ ...prevState, searchLoading: false }));
    }
  };

  const onSearch = () => {
    handleSearchParts();
    setToggleFilter(false);
  };

  const onClear = () => {
    setState((prevState) => ({
      ...prevState,
      carBrand: null,
      carModel: null,
      yearFrom: null,
      yearTo: null,
      carPart: null,
    }));
  };

  const customStyles = {
    control: (provided: any) => ({
      ...provided,
      borderRadius: "0.375rem",
      border: "1px solid #e2e8f0",
      boxShadow: "none",
      "&:hover": {
        border: "1px solid #cbd5e0",
      },
    }),
    placeholder: (provided: any) => ({
      ...provided,
      color: "#a0aec0",
    }),
  };
  
  const addPartToCart = useBuyerStore((state) => state.addToCart);
  
  const cartItems = useBuyerStore((state) => state.cart);

  return (
    <div className="flex flex-col items-center h-full bg-orange-500">
      <div className="w-full max-w-md flex-grow flex flex-col items-center p-4 overflow-hidden h-full">
        <div className="mb-6 mt-4">
          <img
            src="/logoprojeto.png"
            alt="Logo"
            className="rounded-full w-24 h-24"
          />
        </div>
        <h2 className="text-2xl font-normal text-center text-white mb-6">
          Busque sua peça
        </h2>
        <div className="w-full max-w-md bg-white rounded-lg shadow-lg p-6 flex flex-col flex-grow overflow-hidden">
          {!toggleFilter ? (
            <div className="flex justify-end items-center">
              <div onClick={() => setToggleFilter(!toggleFilter)}>
                <Tune
                  className="text-accent cursor-pointer"
                  style={{ fontSize: 32 }}
                />
              </div>
            </div>
          ) : (
            <div>
              <Select
                value={state.carBrand}
                onChange={(value) =>
                  onSelectedBrand(value as { value: string; label: string })
                }
                options={state.brandOptions}
                placeholder="Marca veículo..."
                styles={customStyles}
                loadingMessage={() => "Carregando marcas..."}
                isLoading={state.brandLoading}
                isDisabled={!state.brandOptions.length}
              />
              <Select
                value={state.carModel}
                onChange={(value) =>
                  setState((prevState) => ({
                    ...prevState,
                    carModel: value as { value: string; label: string },
                  }))
                }
                options={state.modelOptions}
                placeholder="Modelo veículo..."
                styles={customStyles}
                loadingMessage={() => "Carregando modelos..."}
                isLoading={state.modelLoading}
                isDisabled={
                  !state.carBrand?.value || !state.modelOptions.length
                }
                className="mt-4"
              />
              <Select
                value={state.carPart}
                onChange={(value) =>
                  setState((prevState) => ({
                    ...prevState,
                    carPart: value as { value: string; label: string },
                  }))
                }
                options={state.partOptions}
                placeholder="Peça do veículo..."
                styles={customStyles}
                loadingMessage={() => "Carregando peças..."}
                isLoading={state.partLoading}
                className="mt-4"
              />
              <div className="flex space-x-2 mt-4">
                <Select
                  value={state.yearFrom}
                  onChange={(value) =>
                    setState((prevState) => ({
                      ...prevState,
                      yearFrom: value as { value: string; label: string },
                    }))
                  }
                  options={years}
                  placeholder="De:"
                  styles={customStyles}
                  className="w-1/2"
                />
                <Select
                  value={state.yearTo}
                  onChange={(value) =>
                    setState((prevState) => ({
                      ...prevState,
                      yearTo: value as { value: string; label: string },
                    }))
                  }
                  options={years}
                  placeholder="Até:"
                  styles={customStyles}
                  className="w-1/2"
                />
              </div>
              <button
                type="button"
                onClick={onSearch}
                className="w-full bg-orange-500 text-white py-2 rounded-md hover:bg-orange-600 transition mt-4"
              >
                Buscar
              </button>
              <button
                type="button"
                onClick={onClear}
                className="w-full bg-gray-300 text-gray-700 py-2 rounded-md hover:bg-gray-400 transition mt-2"
              >
                Limpar
              </button>
            </div>
          )}
          {state.searchLoading ? (
            <div className="flex justify-center my-8 text-orange-500">
              <CircularProgress />
            </div>
          ) : (
            <div className="w-full flex-grow overflow-y-auto no-scrollbar mt-4 h-full">
              {!state.partsResult.length ? (
                <div className="flex flex-col items-center justify-center h-full text-center text-gray-500">
                  <HourglassEmpty style={{ fontSize: 48 }} />
                  <span>Realize uma busca para ver os resultados</span>
                </div>
              ) : (
                <>
                  {state.partsResult.map((part, index) => (
                    <CarCardSearch
                      index={index}
                      key={index}
                      alreadyInCart={cartItems.some((item) => item.id === part.id)}
                      model={part.car.model.enumerator}
                      name={part.part_type.enumerator}
                      onClick={() =>
                        addPartToCart(part)
                      }
                      price={part.price}
                      year={part.car.release_year}
                    />
                  ))}
                </>
              )}
            </div>
          )}
        </div>
      </div>
    </div>
  );
};

export default VehiclePartSearch;
