//eslint-disable @typescript-eslint/no-explicit-any
import { useEffect, useState } from "react";
import CarForm, { CarData } from "../../components/CarForm";
import PieceForm from "../../components/PieceForm";
import AddPieceButton from "../../components/AddPieceButton";
import Button from "../../components/Button";
import { useNavigate } from "react-router-dom";
import { z } from "zod";
import BackButton from "../../components/BackButton";
import api from "../../services/api";
import { DecodedToken } from "../../dtos/decodedToken";
import { jwtDecode } from "jwt-decode";

interface optionDto {
  value: string;
  label: string;
}

const pieceSchema = z.object({
  nome: z
    .number()
    .refine(value => value !== 0, { message: "Nome da peça é obrigatório" }),
  preco: z.string().min(1, { message: "Preço da peça é obrigatório" }),
  observacao: z.string().optional(),
});


const carSchema = z.object({
  marca: z.union([z.string(), z.number()]).refine(value => Number(value) !== 0, { message: "Marca é obrigatória" }),
  modelo: z.union([z.string(), z.number()]).refine(value => Number(value) !== 0, { message: "Modelo é obrigatório" }),
  ano: z.string().min(1, { message: "Ano é obrigatório" }),
  valor: z.string().min(1, { message: "Valor é obrigatório" }),
  data_de_compra: z
    .string()
    .min(1, { message: "Data de compra é obrigatória" }),
});

export interface PieceData {
  nome: string;
  preco: string;
  id: string;
  observacao: string;
  part_status: number;
  [key: string]: string | number;
}


const CadastroCarro = () => {

  const [carData, setCarData] = useState<CarData>({ marca: "", ano: "", valor: "", modelo: "", data_de_compra: "" });
  const [pieceForms, setPieceForms] = useState<PieceData[]>([{ nome: "", preco: "", id: "", observacao: "",  part_status: 1 }]);
  const [carErrors, setCarErrors] = useState<any>({});
const [pieceErrors, setPieceErrors] = useState<any[]>([]);

  const navigate = useNavigate();

  const handleCarInputChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    const { name, value } = e.target;
    setCarData({
      ...carData,
      [name]: value,
    });
  };

  const [carBrandSelected, setCarBrandSelected] = useState("");
  const [carBrand, setCarBrand] = useState<optionDto[] | []>([]);
  const [carPart, setCarPart] = useState<optionDto[] | []>([]);
  const [carModel, setCarModel] = useState<optionDto[] | []>([]);

  useEffect(() => {
    const fetchBrands = async () => {
      const data = await api.get(`/brand`).then((response) => response.data).catch((error) => console.error('Error fetching brands: ', error))
      
      if (!data) {
        return;
      }

      const brandAdated = data.map(
        (item: { enumerator: string; id: string }) => {
          return {
            label: item.enumerator,
            value: item.id,
          };
        }
      );

      setCarBrand(brandAdated);
    };

    const fetchParts = async () => {
      const data = await api
        .get(`/part`)
        .then((response) => response.data)
        .catch((error) => console.error("Error fetching parts: ", error));

      if (!data) {
        return;
      }

      const partAdated = data.map(
        (item: { enumerator: string; id: string }) => {
          return {
            label: item.enumerator,
            value: item.id,
          };
        }
      );

      setCarPart(partAdated);
    };

    fetchBrands();
    fetchParts();
  }, []);

  useEffect(() => {
    const fetchModel = async () => {
      const data = await api
        .get(`/brand/${carBrandSelected}/model`)
        .then((response) => response.data)
        .catch((error) => console.error("Error fetching models: ", error));

      if (!data) {
        return;
      }

      const modelAdated = data.map(
        (item: { enumerator: string; id: string }) => {
          return {
            label: item.enumerator,
            value: item.id,
          };
        }
      );

      setCarModel(modelAdated);
    };
    if (carBrandSelected) {
      fetchModel();
    }
  }, [carBrandSelected]);

  const handleAddPieceClick = () => {
    setPieceForms([...pieceForms, { nome: "", preco: "", id: "", observacao: "", part_status: 1 }]);
    setPieceErrors([...pieceErrors, {}]);
  };

  const handlePieceInputChange = (
    index: number,
    e: React.ChangeEvent<HTMLInputElement>
  ) => {
    const { name, value } = e.target;
    const newPieceForms = [...pieceForms];
    newPieceForms[index][name as keyof PieceData] = value;
    setPieceForms(newPieceForms);
  };

  const handleDeletePiece = (index: number) => {
    if (pieceForms.length <= 1) return;
    const newPieces = [...pieceForms];
    newPieces.splice(index, 1);
    setPieceForms(newPieces);
  };

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();

    const carValidation = carSchema.safeParse(carData);
    if (!carValidation.success) {
      setCarErrors(carValidation.error.flatten().fieldErrors);
      
      const readableErrors = Object.fromEntries(Object.entries(carValidation.error.flatten().fieldErrors).map(([key, value]) => [key, value.join(', ')]));
      console.log(readableErrors)

      return;
    } else {
      setCarErrors({});
    }


    const newPieceErrors = [...pieceErrors];
    let hasPieceError = false;
    pieceForms.forEach((piece, index) => {
      const pieceValidation = pieceSchema.safeParse(piece);
      if (!pieceValidation.success) {
        hasPieceError = true;
        newPieceErrors[index] = pieceValidation.error.flatten().fieldErrors;
      } else {
        newPieceErrors[index] = {};
      }
    });
    if (hasPieceError) {
      setPieceErrors(newPieceErrors);
      return;
    } else {
      setPieceErrors([]);
    }

    const formattedData = {
      model: {
        model_id: Number(carData.modelo),
      },
      release_year: carData.ano,
      cost: parseFloat(carData.valor),
      purchase_date: carData.data_de_compra,
      parts: pieceForms.map((piece) => ({
        part_type_id: Number(piece.nome),
        description: piece.observacao,
        price: parseFloat(piece.preco),
      })),
    };

    try {
      const token = localStorage.getItem("token");

      if (!token) {
        throw new Error("No token found");
      }

      const decoded = jwtDecode<DecodedToken>(token);

      const response = await api.post(
        `/seller/${decoded.id}/cars`,
        formattedData
      );

      if (response.status !== 201) {
        throw new Error("Failed to submit data");
      }

      navigate("/home", {
        state: { createdCar: true }
      });

      // Handle successful submission
    } catch (error) {
      console.error("Error submitting data:", error);
      // Handle error
    }
  };

  return (
    
    <div className="relative flex flex-col justify-center items-center min-h-screen bg-[#FFF4EA]">
      <div className="absolute top-6 left-6">
        <BackButton showModalOnBack={true} />
      </div> 
      <div className="flex justify-center mb-6">
        <img src="/logo.svg" alt="Logo" className="w-20 h-auto" />
      </div>
      
      <div className="w-full max-w-7xl py-5">
        <div className="bg-white p-8 rounded-lg shadow-lg max-h-[75vh] overflow-y-scroll">
          <div className="w-full max-w-7xl py-5">
            <form onSubmit={handleSubmit}>
            <h2 className="text-2xl font-bold text-black mb-6">
              Adicionar Carro
            </h2>
              <CarForm
                carData={carData}
                carErrors={carErrors}
                carBrand={carBrand}
                carModel={carModel}
                onCarInputChange={handleCarInputChange}
                onBrandSelect={(option: optionDto) => setCarBrandSelected(option.value)}
              />
              <h2 className="text-2xl font-bold text-black mb-4 mt-6">
              Adicionar Peças
              </h2>
              {pieceForms.map((pieceForm, index) => (
                <PieceForm
                  key={index}
                  index={index}
                  form={pieceForm}
                  errors={pieceErrors[index]}
                  carPart={carPart}
                  handleInputChange={handlePieceInputChange}
                  handleDelete={handleDeletePiece}
                />
              ))}
              
              
            </form>
            <AddPieceButton onClick={handleAddPieceClick} />
          </div>
          
        </div>
        <div className="flex justify-end mt-8">
                <div className="w-80">
                  <Button
                    isFluid={true}
                    customClass="bg-amber-500 hover:bg-amber-600 text-white transition-all duration-300 shadow-md hover:shadow-lg transform hover:translate-x-1"
                    onClick={handleSubmit}
                  >
                    Cadastrar Veículo
                  </Button>
                </div>
              </div>
      </div>
    </div>
  );
};

export default CadastroCarro;