import React, { useContext, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import Input from '../../components/Input';
import Button from '../../components/Button';
import BackButton from '../../components/BackButton'; 
import { useForm } from 'react-hook-form';
import { zodResolver } from '@hookform/resolvers/zod';
import { z } from 'zod';
import { FormDataContext } from '../../contexts/FormDataContext';
import api from '../../services/api';

interface FormData {
  nomeRepresentante: string;
  cpf: string;
  email: string;
  telefone: string;
  senha: string;
  confirmarSenha: string;
}

const representanteSchema = z
  .object({
    nomeRepresentante: z
      .string()
      .min(3, { message: 'Nome do Representante deve ter pelo menos 3 caracteres' }),
    cpf: z.string().length(11, { message: 'CPF deve ter 11 dígitos' }),
    email: z.string().email({ message: 'Email inválido' }),
    telefone: z.string().regex(/^\d{10,11}$/, { message: 'Telefone inválido' }),
    senha: z.string().min(6, { message: 'Senha deve ter pelo menos 6 caracteres' }),
    confirmarSenha: z.string().min(6, { message: 'Confirme a senha' }),
  })
  .refine((data) => data.senha === data.confirmarSenha, {
    path: ['confirmarSenha'],
    message: 'As senhas não coincidem',
  });

const CadastroRepresentante: React.FC = () => {
  const navigate = useNavigate();
  const { cadastroRepresentante, cadastroFerroVelho, setCadastroRepresentante, clearData } = useContext(FormDataContext);
  const [isLoading, setIsLoading] = useState<boolean>(false);

  const {
    register,
    handleSubmit,
    formState: { errors },
  } = useForm<FormData>({
    resolver: zodResolver(representanteSchema),
    defaultValues: cadastroRepresentante,
  });

  const onSubmit = async (data: FormData) => {

    const formattedData = {
      user: {
        username: data.nomeRepresentante,
        email: data.email,
        password: data.senha,
        document_number: data.cpf,
      },
      seller_code: cadastroFerroVelho.cdv,
      company_name: cadastroFerroVelho.nomeEmpresa,
      trading_name: cadastroFerroVelho.nomeFantasia,
      document_number: cadastroFerroVelho.cnpj,
      email: cadastroFerroVelho.email,
      phone_country_code: '+55',
      phone_area_code: cadastroFerroVelho.telefone.slice(0,2),
      phone_number: cadastroFerroVelho.telefone.slice(2,11),
      street: cadastroFerroVelho.endereco,
      neighborhood: cadastroFerroVelho.bairro,
      number: cadastroFerroVelho.numero,
      city: cadastroFerroVelho.cidade,
      state: cadastroFerroVelho.estado,
      complement: cadastroFerroVelho.complemento,
      postal_code: cadastroFerroVelho.cep,
    };
  
    try {
      setIsLoading(true);
      await api.post(`/seller`, formattedData);
      clearData()
      navigate('/admin', {
        state: { createdSeller: true }
      });
    } catch (error) {
      console.log('error: ', error)
    } finally {
      setIsLoading(false); 
    }
  
    setCadastroRepresentante(data);
  };

  const handleConfirmBack = () => {
    setCadastroRepresentante({});
    navigate(-1); 
  };

  return (
    <div className="relative flex flex-col justify-center items-center min-h-screen bg-[#FFF4EA]">
      <div className="absolute top-6 left-6">
        <BackButton showModalOnBack={true} onConfirmBack={handleConfirmBack} />
      </div>

      <div className="flex justify-center mb-6">
        <img src="/logo.svg" alt="Logo" className="w-20 h-auto" />
      </div>
      <div className="w-full max-w-4xl py-5">
        <div className="bg-white p-8 rounded-lg shadow-lg">
          <form id="cadastro-form">
            <h2 className="text-2xl font-bold text-gray-800 mb-6">Cadastro Representante</h2>
            <h3 className="text-lg font-semibold mb-4">
              Informações do representante da empresa
            </h3>
            <div className="grid grid-cols-2 gap-x-12 gap-y-6">
              <div>
                <div className="space-y-4">
                  <Input
                    label="Nome Representante"
                    name="nomeRepresentante"
                    placeholder="Digite o Nome do Representante"
                    register={register('nomeRepresentante')}
                    error={errors.nomeRepresentante?.message}
                    autoComplete="name"
                  />
                  <Input
                    label="CPF"
                    name="cpf"
                    placeholder="Digite o CPF"
                    register={register('cpf')}
                    error={errors.cpf?.message}
                    autoComplete="off"
                  />
                  <Input
                    label="Email"
                    name="email"
                    type="email"
                    placeholder="Digite o Email"
                    register={register('email')}
                    error={errors.email?.message}
                    autoComplete="email"
                  />
                </div>
              </div>
              <div>
                <div className="space-y-4">
                  <Input
                    label="Telefone"
                    name="telefone"
                    type="tel"
                    placeholder="Digite o Telefone"
                    register={register('telefone')}
                    error={errors.telefone?.message}
                    autoComplete="tel"
                  />
                  <Input
                    label="Senha"
                    name="senha"
                    type="password"
                    placeholder="Digite a Senha"
                    register={register('senha')}
                    error={errors.senha?.message}
                    icon="/key.svg"
                    autoComplete="new-password"
                  />
                  <Input
                    label="Confirmar Senha"
                    name="confirmarSenha"
                    type="password"
                    placeholder="Confirme a Senha"
                    register={register('confirmarSenha')}
                    error={errors.confirmarSenha?.message}
                    icon="/key.svg"
                    autoComplete="new-password"
                  />
                </div>
              </div>
            </div>
          </form>
        </div>
        <div className="flex justify-end mt-8">
          <div className="w-80">
            <Button
              isFluid={true}
              onClick={handleSubmit(onSubmit)}
              customClass="bg-amber-500 hover:bg-amber-600 text-white transition-all duration-300 shadow-md hover:shadow-lg transform hover:translate-x-1"
            >
              {isLoading ? 'Carregando' : 'Cadastrar usuário'}
            </Button>
          </div>
        </div>
      </div>
    </div>
  );
};

export default CadastroRepresentante;
